# == Schema Information
#
# Table name: users
#
#  id                     :integer          not null, primary key
#  email                  :string           default(""), not null
#  encrypted_password     :string           default(""), not null
#  reset_password_token   :string
#  reset_password_sent_at :datetime
#  remember_created_at    :datetime
#  sign_in_count          :integer          default("0"), not null
#  current_sign_in_at     :datetime
#  last_sign_in_at        :datetime
#  current_sign_in_ip     :string
#  last_sign_in_ip        :string
#  created_at             :datetime         not null
#  updated_at             :datetime         not null
#  provider               :string           default("email"), not null
#  uid                    :string           default(""), not null
#  confirmation_token     :string
#  confirmed_at           :datetime
#  confirmation_sent_at   :datetime
#  unconfirmed_email      :string
#  tokens                 :text
#  name                   :string
#  avatar_file_name       :string
#  avatar_content_type    :string
#  avatar_file_size       :integer
#  avatar_updated_at      :datetime
#  bookmarks_count        :integer          default("0")
#  user_category_id       :integer
#  score                  :float            default("0.0")
#  follows_count          :integer          default("0")
#

class User < ActiveRecord::Base
  include Followable
  include Reputationable
  include Tagger

  # Include default devise modules. Others available are:
  # :confirmable, :lockable, :timeoutable and :omniauthable
  devise :database_authenticatable, :registerable,
         :recoverable, :rememberable, :trackable, :validatable,
         :omniauthable

  include DeviseTokenAuth::Concerns::User

  has_and_belongs_to_many :roles
  has_and_belongs_to_many :institutions

  has_many :bookmarks, as: :bookmarkable
  has_many :bookmark_collections, through: :bookmarks, source: :bookmarkable, source_type: 'Collection'
  has_many :bookmark_learning_objects, through: :bookmarks, source: :bookmarkable, source_type: 'LearningObject'
  has_many :bookmarks
  has_many :collections, as: :owner
  has_many :learning_objects, as: :publisher
  has_many :views
  has_many :downloads
  has_many :likes
  has_many :shares
  has_many :follows
  has_many :reviews
  has_many :applications

  after_create :default_role

  has_attached_file :avatar, styles: {medium: "300x300>", thumb: "60x60>"}, default_url: lambda { |image| ActionController::Base.helpers.asset_path('user-anon.png') }
  validates_attachment_content_type :avatar, content_type: /\Aimage\/.*\Z/
  validates :terms_of_service, acceptance: true

  searchkick language: 'brazilian', match: :word_start, searchable: [:name], callbacks: :async

  acts_as_paranoid
  has_paper_trail

  def search_data
    {
        name: name,
        score: score
    }
  end

  def is_admin?
    roles.each do |role|
      return true if role.name == "admin"
    end
    false
  end

  def associated_collections
    c = collections.blank? ? [] : collections.includes(:owner)
    c + Collection.where(owner: institutions).includes(:owner)
  end

  # ~~~~ followable actions ~~~~ #
  # An user can follow anothers users and collections
  # Examples:
  # current_user.follow(User.find_by_email('john@revelator.com'))
  # current_user.follow(Collection.find(143))
  def follow(followable)
    Follow.create(user: self, followable: followable)
  end

  # Examples:
  # current_user.unfollow(User.find_by_email('john@revelator.com'))
  # current_user.unfollow(Collection.find(143))
  def unfollow(followable)
    Follow.where(user: self, followable: followable).destroy_all
  end

  # An user can see if is following another user
  # Examples:
  # current_user.following?(User.find_by_email('john@revelator.com')) => true
  # current_user.following?(Collection.find(143)) => true
  def following?(followable)
    !follows.where(followable_id: followable.id, followable_type: followable.class.name).blank?
  end

  # This function permits see what person is following.
  # For current user, list all users and all collections. But,
  # if, isn't current user, list only users, and public collections which this person follows.
  def watching(followable_type, is_current_user = true)
    f = Follow.where(user_id: id, followable_type: followable_type).all

    # If type collection, and isn't current_user, so should list only public collections
    if followable_type.to_s == 'Collection' && !is_current_user
      f = Follow.joins('INNER JOIN collections ON follows.followable_id = collections.id').where(
        "follows.user_id = #{id} " \
        'AND follows.followable_type = \'Collection\' ' \
        'AND collections.id = follows.followable_id ' \
        'AND collections.privacy = \'public\''
      )
    end

    f
  end

  # ~~~~ end followable actions ~~~~ #

  ## get all activities from user
  # return an array of PublicActivity::Activity
  def activities
    condition = '(owner_type = :type AND owner_id = :id) OR (recipient_type = :type AND recipient_id = :id)'
    PublicActivity::Activity.order('created_at DESC').where(condition, {type: object.class.to_s, id: object.id}).all
  end

  private

  def default_role
    roles << Role.find_by_name('teacher')
  end

  def self.from_omniauth(access_token)
    data = access_token.info

    if access_token.provider == 'twitter'
      user_email = "#{access_token.uid}@twitter.com"
    else
      user_email = data['email']
    end

    user = User.where(email: user_email).first

    unless user
      user = User.create(
          name: data['name'],
          email: user_email,
          provider: access_token.provider,
          uid: access_token.uid,
          password: Devise.friendly_token[0, 20]
      )
    end
    user
  end

end
